# main.py

########################################################################
#  SdiQt -- SDictionary viewer based on Qt library
#  Copyright (C) 2006 Dmitri Brechalov
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#  The full license is in the file COPYING, distributed as part of
#  this software.
########################################################################

import utils
import os.path
from qt import *
if utils.isQtopia():
    from qtpe import *
    from filedialog import FileDialog
    from fontdialog import FontDialog

import version
import sdict
    
from browser import Browser

def prepareArticle(title, text):
    import sre
    text = sre.sub('<r>([^<]+)</r>',
                   '<a href="\\1">\\1</a>',
                   text)
    text = text.replace('<l>', '<ul>')
    text = text.replace('</l>', '</ul>')
    text = text.replace('<t>', '<tt><b>/')
    text = text.replace('</t>', '/</b></tt>')
    text = text.replace('<f>', '<tt>')
    text = text.replace('</f>', '</tt>')
    return '<h3>%s</h3>\n\n%s' % (title, text)

class mainForm(QMainWindow):
    def __init__(self,parent = None):
        QMainWindow.__init__(self, parent, '', 0)

        self._error = None
        self.dct = None
        self.fileName = None
        self.dictHistory = {}
        self.history = []
        self.timer = None
        self.nosearch = False
        self.config = utils.SimpleConfig('~/.sdiqt')

        self.setCaption(version.product + ' ' + version.version)
        if utils.isQtopia():
            iconDir = '/opt/QtPalmtop/pics'
        else:
            iconDir = utils.getProgramPath()
        self.setIcon(QPixmap(os.path.join(iconDir, 'sdiqt.png'), 'PNG'))

        self.setupMainMenu()
        self.setupToolbar()
        self.setupMainArea()
        self.setupSlots()
        self.languageChange()
        self.resize(QSize(640,480).expandedTo(self.minimumSizeHint()))
        self.readConfig()
        try:
            self.openLastDictionary()
        except Exception, details:
#            pass
            self.messageBox('Error opening last dictionary:\n' + str(details), 'error')

    def openLastDictionary(self):
        if self.fileName:
            self.loadDictionary()

    def loadDictionary(self):
        try:
            self.dct = sdict.SDictionary(self.fileName)
        except Exception, details:
            self.messageBox('Cannot open file %s!\n%s' % \
                            (self.fileName, details), 'error')
            self.dct = None
            self.fileName = None
#        self.history = []
        self.updateForm()

    def close(self, alsoDelete=False):
        #### save last dict path
        self.writeConfig()
        self.config.close()
        if self.dct:
            self.dct.close()
        rc = QMainWindow.close(self, alsoDelete)
        return rc

    def readConfig(self):
        self.fileName = self.config.readStr('last')
        fontProps = self.config.readStr('font')
        if fontProps:
            (family, size) = fontProps.split(',')
            font = QFont()
            font.setFamily(family)
            font.setPointSize(int(size))
            self.setupFont(font)
        nkeys = self.config.keys('name')
        nkeys.sort()
        self.dictHistory = {}
        for k in nkeys:
            n = k[4:]
            name = self.config.readStr(k)
            fname = self.config.readStr('file'+n)
            utils.appendHistory(self.dictHistory, self.dictList, fname, name)

    def writeConfig(self):
        self.config.writeStr('last', self.fileName)
        font = self.wordSearch.font()
        fontProps = '%s,%s' % (font.family(), font.pointSize())
        self.config.writeStr('font', fontProps)
        i = 0
        for (name, fname) in self.dictHistory.items():
            self.config.writeStr('name' + str(i), name)
            self.config.writeStr('file' + str(i), fname)
            i += 1
        
    def setupMainMenu(self):
        self.helpAboutAction = QAction(self, "helpAboutAction")
        self.helpAboutQtAction = QAction(self, "helpAboutQtAction")
        self.optionsFont = QAction(self, "optionsFont")
        self.optionsPrefs = QAction(self, "optionsPrefs")
        self.fileOpen = QAction(self, "fileOpen")
        self.fileExit = QAction(self, "fileExit")

        if utils.isQtopia():
            self.menubar = QPEMenuBar(self,"menubar")
        else:
            self.menubar = QMenuBar(self,"menubar")

        self.fileMenu = QPopupMenu(self)
        self.fileOpen.addTo(self.fileMenu)
        self.fileMenu.insertSeparator()
        self.fileExit.addTo(self.fileMenu)
        self.menubar.insertItem("&File",self.fileMenu,0)

        self.optionsMenu = QPopupMenu(self)
        self.optionsFont.addTo(self.optionsMenu)
        self.optionsPrefs.addTo(self.optionsMenu)
        self.menubar.insertItem("&Options",self.optionsMenu,3)

        self.helpMenu = QPopupMenu(self)
        self.helpAboutAction.addTo(self.helpMenu)
        self.helpAboutQtAction.addTo(self.helpMenu)
        self.menubar.insertItem("&?",self.helpMenu,4)

    def setupToolbar(self):
        if utils.isQtopia():
            self.toolBar = QPEToolBar(self)
        else:
            self.toolBar = QToolBar(self)
        try:
            self.setToolBarsMovable(False)
        except:
            pass

        self.dictLbl = QLabel(self.toolBar, "dictLbl")
        self.dictLbl.setText("Dictionary: ")
        self.dictList = QComboBox(0,self.toolBar,"dictList")
        self.dictList.setMinimumSize(QSize(150,0))
        self.toolBar.setStretchableWidget(self.dictList)
        self.btnBack = QPushButton(self.toolBar, 'btnBack')
        self.btnBack.setText('<=')
        self.btnBack.setEnabled(False)

    def setupMainArea(self):
        w = QWidget(self, "w")
        self.setCentralWidget(w)
        mainFormLayout = QHBoxLayout(w,1,2,"mainFormLayout")
        lw = QVBox(w, "lw")
#        lw.setMaximumSize(self.width() / 4, lw.maximumHeight())
        lw.setMaximumSize(200, lw.maximumHeight())
        lw.setSizePolicy(QSizePolicy(QSizePolicy.Fixed, QSizePolicy.Expanding))
        mainFormLayout.addWidget(lw)
        self.wordSearch = QComboBox(lw, "wordSearch")
        self.wordSearch.setEditable(True)
        self.wordList = QListBox(lw,"wordList")
        rightLayout = QVBoxLayout(None,1,2,"rightLayout")
        self.textView = Browser(w, "textView")
        rightLayout.addWidget(self.textView)
        mainFormLayout.addLayout(rightLayout)
        self.wordSearch.setFocus()

    def setupSlots(self):
        #### main menu
        self.connect(self.helpAboutAction, SIGNAL("activated()"),
                     self.helpAbout)
        self.connect(self.helpAboutQtAction, SIGNAL("activated()"),
                     self.helpAboutQt)
        self.connect(self.optionsFont, SIGNAL("activated()"),
                     self.changeFont)
        self.connect(self.optionsPrefs, SIGNAL("activated()"),
                     self.changePreferences)
        self.connect(self.fileOpen, SIGNAL("activated()"),
                     self.openDictionary)
        self.connect(self.fileExit, SIGNAL("activated()"),
                     self.close)
        #### searching
        self.connect(self.wordSearch.lineEdit(),
                     SIGNAL("returnPressed()"),
                     self.search)
        self.connect(self.wordSearch.lineEdit(),
                     SIGNAL("textChanged(const QString &)"),
                     self.startSearch)
        self.connect(self.wordList,
                     SIGNAL("highlighted(int)"),
                     self.activateWordList)
        #### hypertext capabilities
        self.connect(self.textView,
                     PYSIGNAL("hyperlinkClicked()"),
                     self.hyperlinkClicked)
        self.connect(self.btnBack,
                     SIGNAL("clicked()"),
                     self.backward)
        #### other commands
        self.connect(self.dictList,SIGNAL("activated(const QString &)"),self.dictChanged)

    def languageChange(self):
        self.helpAboutAction.setText(self.tr("About"))
        self.helpAboutAction.setMenuText(self.tr("&About"))

        self.helpAboutQtAction.setText(self.tr("About Qt"))
        self.helpAboutQtAction.setMenuText(self.tr("About &Qt"))

        self.fileOpen.setText(self.tr("Open dictionary"))
        self.fileOpen.setMenuText(self.tr("&Open"))
        self.fileOpen.setAccel(Qt.CTRL+Qt.Key_O)

        self.optionsFont.setText(self.tr("Change Font"))
        self.optionsFont.setMenuText(self.tr("&Font"))
        self.optionsPrefs.setText(self.tr("Preferences"))
        self.optionsPrefs.setMenuText(self.tr("&Preferences"))

        self.fileExit.setText(self.tr("Exit"))
        self.fileExit.setMenuText(self.tr("E&xit"))
        self.fileExit.setAccel(Qt.CTRL+Qt.Key_Q)

    def messageBox(self, msg, title="Warning"):
        QMessageBox.warning(self, title, msg)

    def notImplemented(self, feature):
        self.messageBox('Sorry, %s not implemented yet!' % feature)

    def helpAbout(self):
        QMessageBox.about(self, "About",
                          "<h2>" + version.product + "</h2>" + \
                          "<h3>" + version.description + "</h3>" + \
                          "<p>Version %s<p>" % \
                          (version.version) + \
                          '<nobr>' + version.copyright + '</nobr>' + \
                          '<p><small>' + version.notice + '</small>')

    def helpAboutQt(self):
        QMessageBox.aboutQt(self, "About Qt")

    def changeFont(self):
        if utils.isQtopia():
            dlg = FontDialog(self, self.wordSearch.font())
            if dlg.exec_loop():
                font = dlg.selectedFont()
                self.setupFont(font)
        else:
            (font, ok) = QFontDialog.getFont(self.wordSearch.font(), self)
            if ok:
                self.setupFont(font)
            

    def setupFont(self, font):
        self.wordSearch.setFont(font)
        self.wordList.setFont(font)
        self.textView.setFont(font)

    def changePreferences(self):
        self.notImplemented('Preferences Dialog')

    def dictChanged(self, txt):
        self.fileName = self.dictHistory[str(txt.utf8())]
        self.loadDictionary()
        self.wordSearch.setFocus()

    def openDictionary(self):
        import os.path
        if self.fileName:
            dirName = os.path.split(self.fileName)[0]
        else:
            dirName=os.path.expanduser('~')
        if utils.isQtopia():
            dlg = FileDialog(self, 'OpenFile', 'Open Dictionary',
                             dirName = dirName,
                             filter='*.dct')
        else:
            dlg = QFileDialog(dirName, '*.dct', self, 'OpenFile')
        if dlg.exec_loop() and dlg.selectedFile().utf8():
            self.fileName = str(dlg.selectedFile().utf8())
            self.loadDictionary()

    def updateForm(self):
        if self.dct is None:
            self.dictList.setCurrentItem(-1)
        else:
            utils.appendHistory(self.dictHistory, self.dictList,
                                self.fileName, self.dct.title)
            self.showDictInfo()
        if str(self.wordSearch.currentText().utf8()):
            self.doSearch()
        else:
            self.wordList.clear()
            self.btnBack.setEnabled(len(self.history)>1)
        
    def activateWordList(self, idx):
        self.wordList.setSelected(idx, True)
        self.showWord(str(self.wordList.currentText().utf8()))

    def search(self):
        self.wordList.setFocus()

    def startSearch(self):
        if self.nosearch:
            return
        if self.timer:
            self.killTimer(self.timer)
        self.timer = self.startTimer(600)

    def timerEvent(self, event):
        if event.timerId() == self.timer:
            self.doSearch()
        else:
            QWidget.timerEvent(event)

    def doSearch(self):
        if self.timer:
            self.killTimer(self.timer)
            self.timer = None
        word = str(self.wordSearch.currentText().utf8())
        self.wordList.clear()
        if not word:
            self.showDictInfo()
            return
        for item in self.dct.get_word_list(word):
            self.wordList.insertItem(QString.fromUtf8(item))
        if not self.history or self.history[-1] != word:
            self.history.append(word)
        self.btnBack.setEnabled(len(self.history)>1)

    def showWord(self, word):
        self.nosearch = True
        self.wordSearch.setEditText(QString.fromUtf8(word))
        article = prepareArticle(word, self.dct.lookup(word))
        self.textView.setText(QString.fromUtf8(article))
        self.nosearch = False

    def hyperlinkClicked(self, word):
        self.wordSearch.setEditText(word)

    def backward(self):
        if len(self.history) > 1:
            word = self.history.pop()   # the current page
            word = self.history.pop()
            self.wordSearch.setEditText(QString.fromUtf8(word))
            self.doSearch()

    def showDictInfo(self):
        info = '''<table border="0">
        <tr><td colspan="2"><b>%s</b></td></tr>
        <tr><td>Version: </td><td><b>%s</b></td></tr>
        <tr><td>Copyright: </td><td><b>%s</b></td></tr>
        <tr><td>Filename: </td><td><b>%s</b></td></tr>
        <tr><td>Total </td><td><b>%s</b> words</td></tr>
        </table>''' % (self.dct.title, self.dct.version,
                       self.dct.copyright, self.fileName,
                       self.dct.header.num_of_words)
        self.textView.setText(QString.fromUtf8(info))

# end of main.py
